/*	controller.js
	Another class name we could use here would be Router
*/

const
	UrlWrapper = require('url-wrapper').UrlWrapper,
	TemperatureConverter = require('./model/temperature-converter').TemperatureConverter,			
	LengthConverter = require('./model/length-converter').LengthConverter,	
	EnglishToSpanishTranslator = require('./model/english-translator').EnglishToSpanishTranslator,	
	statusCodes = require('http-status-codes'),
	log = console.log,
	htmlContentType = {'Content-Type': 'text/html'},
	plainType = {"Content-Type": "text/plain"},	
	br = '<br>';

//=====================================	

class Controller {
	
	constructor(request, response) {
		this.url = UrlWrapper.fromRequest(request);
		this.request = request;
		this.response = response;
	}
	
	processRequest() {
		this.logRequestInfo();
		this.route();
	}
	
	route() {
		const path = this.url.path();
		if (path === '/convert/temperature')
			this.convert(TemperatureConverter);
		else if (path === '/convert/length')
			this.convert(LengthConverter);		
		else if (path === '/translate/english')
			this.translateEnglish();	
		else
			log(`URL Path not handled: ${path}`);			
	}
	
	//-------------------------------------
	//Calcs
	
	convert(converterClass) {
		const 
			query = this.url.query(),
			unit = query.unit,
			value = query.value;		
		if (!this.validateParam('unit', unit) || !this.validateParam('value', value))
			return;						
		const converter = new converterClass(unit, value);
		this.serveConversionResults(converter.getResults());
	}
	
	translateEnglish() {
		const englishWord = this.url.query().word;	
		if (!this.validateParam('word', englishWord))
			return;
		const translator = new EnglishToSpanishTranslator(englishWord);
		this.serveTranslationResults(translator.getResults());
	}	

	//-------------------------------------
	//validation - private
	
	validateParam(paramName, value) {
		if (value === undefined) {
			const res = this.response;
			res.writeHead(statusCodes.BAD_REQUEST, plainType);
			res.end('ERROR: ' + `Missing query param "${paramName}"`);
			return false;
		}
		return true; //ok
	}
	
	//-------------------------------------
	//serving data - private	
	
	serveConversionResults(arrayResults) {
		//serve result to client/consumer
		const res = this.response;
		res.writeHead(statusCodes.OK, htmlContentType);		
		//result is an array
		for (let pair of arrayResults)
			res.write(`${pair.value} ${pair.unit}` + br);
		res.end();
	}
	
	serveTranslationResults(arrayResults) {
		//serve result to client/consumer
		const res = this.response;
		res.writeHead(statusCodes.OK, htmlContentType);		
		//result is an array
		for (let pair of arrayResults)
			res.write(`${pair.english} -> ${pair.spanish}` + br);
		res.end();
	}	
	
	//-------------------------------------
	//helpers - private		
	
	logRequestInfo() {
		//dump request info to console
		log('received url: ' + this.request.url);	
		log('url path: ' + this.url.path());
		//stringify converts to user-friendly format
		log('url query: ' + JSON.stringify(this.url.query()));			
	}	
	
}
	
//=====================================	

exports.Controller = Controller;	
	
	
	
